package mtv.core;

//import static de Java 5 permet d'importer les attributs et autres methodes 
//statiques pour ne plus avoir  faire appel  NomClasse.trucstatique
//Ainsi si j'ai besoin de Lesson.ENG et que je l'importe statiquement, je n'aurai qu' appeler ENG dans
//je n'aurai qu' appeler ENG dans le reste du code.

import static mtv.core.Lesson.ENG; 
import static mtv.core.Lesson.FR; 

import java.net.URL;
import java.util.Observable;
import java.util.ArrayList;
import java.util.Iterator;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

/**
 * Classe reprsentant un test. Elle sert de modle dans l'architecture MVC(Model - View - Controller).
 * Un test est compos de plusieurs leons qui peuvent tre ou non incluse dans le test ("coches" par l'utilisateur). 
 * Si une leon est coche, les mots de la leons pourront tre soumis  l'utilisateur.
 * Le score est affich  la fin d'un test et correspond au nombre de mots de vocabulaires maitriss. 

 *
 * @author Christopher Henard
 * @version 26/10/2008
 */

public class Test extends Observable{

    //CONSTANTES--------------------------------------------------
    public static final char wordSeparator = '=';
    public static final String lessonSeparator = "";
    public static final String errorMsg = " --- WRONG : The correct answer is ";



    //CHAMPS------------------------------------------------------
    private ArrayList<Lesson> lessons;
    private ArrayList<Lesson> lessonsChecked;
    private ArrayList<Lesson> lessonsCheckedCopy;
    private int score;
    private int maxScore;
    private int wordsCount;
    private int checkedWordsCount;
    private int asked, errors;
    private String input;
    private String output;
    private int randomWord = 0;
    private int randomLessonIndex = 0;
    private Lesson randomLesson;
    private int lang = 0;
    private boolean finished, eval, reasking;
    private String lesson;



    //CONSTRUCTEURS-----------------------------------------------
    /**
     * Construit un test  partir d'un nombre variable de leons.
     * @param lessons les leons du test (... = "argument variables" de Java 5).
     */
    public Test(Lesson ... lessons){
	this.lessons = new ArrayList<Lesson>();
	lessonsChecked = new ArrayList<Lesson>();
	for (Lesson l : lessons){
	    addLesson(l);
	    wordsCount += l.getLength();
	    setMaxScore(getMaxScore() + l.getLength());
	    if (l.isChecked()){
		addLessonChecked(l);
		checkedWordsCount += l.getLength();
	    }
	}	
	lang = asked = errors = 0;
	setScore(0);
	eval = false;
	reasking = finished = true;
	input = output = lesson = "";
    }

    /**
     * Construit un test  partir des leons contenus dans un fichier texte
     * en utilisant la methode load (voir plus bas).
     *
     * @param fileName le nom du fichier contenant les leons.
     */
    public Test(String fileName){
	lessons = new ArrayList<Lesson>();
	lessonsChecked = new ArrayList<Lesson>();
	lessonsCheckedCopy = new ArrayList<Lesson>();
	setScore(0);
	input = output = lesson = "";
	eval = false;
	reasking = finished = true;
	lang = asked = errors = 0;
	try{
	    load(fileName);
	}	    
	catch (LoadException e) {
	    e.getMessage();
	}
    }



    //METHODES D'OBSERVATION ET DE TRANSFORMATION------------------
    /**
     * Retourne l'ensemble des lessons du test (incluse ou non).
     * @return toutes les leons du test.
     */
    public ArrayList<Lesson> getLessons(){
	return lessons;
    }

    /**
     * Retourne l'ensemble des lessons du test qui seront soumise  l'utilisateur.
     * @return toutes les leons du test qui sont "coches".
     */
    public ArrayList<Lesson> getLessonsChecked(){
	return lessonsChecked;
    }

    /**
     * Retourne une copie l'ensemble des lessons du test qui seront soumise  l'utilisateur.
     * @return toutes les leons du test qui sont "coches".
     */
    public ArrayList<Lesson> getLessonsCheckedCopy(){
	return lessonsCheckedCopy; 
    }

    /**
     * Retourne le nombre de leons.
     * @return le nombre de leons.
     */
    public int getLength() {
	return lessons.size();
    }

    /**
     * Retourne le nombre de leons coches.
     * @return le nombre de leons coches.
     */
    public int getCheckedLength() {
	return lessonsChecked.size();
    }

    /**
     * Retourne le nombre total de mots.
     * @return le nombre total de mots.
     */
    public int getWordsCount(){
	return wordsCount;
    }

    /**
     * Retourne le nombre total de mots des leons coches.
     * @return le nombre total de mots des leons coches.
     */
    public int getCheckedWordsCount(){
	return checkedWordsCount;
    }
	    
    /**
     * Retourne la leon d'indice index.
     * @return la leon dont l'indice est index.
     */
    public Lesson getLesson(int index){
	return lessons.get(index);
    }

    /**
     * Retourne la leon coche d'indice index.
     * @return la leon cochedont l'indice est index.
     */
    public Lesson getLessonChecked(int index){
	return lessonsChecked.get(index);
    }

    /**
     * Retourne true si toutes les leons sont coches.
     * @return true si toutes les leons sont coches.
     */
    public boolean allChecked(){
	boolean b = true;
	for (Lesson l : getLessons()){
	    b = b && l.isChecked();
	}
	return b;
    }

    /**
     * Retourne true si on redemande les mots errons
     * @return true si on redemande les mots errons.
     */
    public boolean getReasking(){
	return reasking;
    }

    /**
     * Retourne le nombre de mot demands.
     * @return le nombre de mot demands. 
     */
    public int getAsked(){
	return asked;
    }

   /**
     * Retourne le nombre d'erreurs.
     * @return le nombre d'erreurs. 
     */
    public int getErrors(){
	return errors;
    }

    /**
     * Retourne le score.
     * @return le score atteint au test.
     */
    public int getScore(){
	return score;
    }

    /**
     * Retourne la langue d'interrogation du test.
     * @return la langue d'interrogation du test.
     */
    public int getLang(){
	return lang ;
    }

    /**
     * Teste si le test est termin.
     * @return true si le test est fini.
     */
    public boolean isFinished(){
       return finished;
    }

    /**
     * Retourne le score maximal.
     * @return le score maximal du test.
     */
    public int getMaxScore(){
	return maxScore;
    }

    /**
     * Retourne la chaine saisie par l'utilisateur.
     * @return la chaine saisie par l'utilisateur dans le champs input.
     */
    public String getInput(){
	return input;
    }

    /**
     * Retourne la chaine  afficher en sortie.
     * @return la chaine  afficher en output.
     */
    public String getOutput(){
	return output;
    }

    /**
     * Teste si la reponse a la question est juste ou non.
     * @return true si la reponse  la question est juste.
     */
    public boolean getEval(){
	return eval;
    }


    /**
     * Retourne la leon.
     * @return la leon sous forme de chane de caractres.
     */
    public String getLesson(){
	return lesson;
    }

    /**
     * Modifie le score.
     * @param score le nouveau score.
     */
    public void setScore(int score){
	this.score = score;
	setChanged();
	notifyObservers();
    }

    /**
     * Modifie le score maximal.
     * @param maxScore le nouveau score maximal.
     */
    public void setMaxScore(){
	maxScore = 0;
	for (Lesson l : getLessonsChecked())
	    maxScore += l.getLength();
	setChanged();
	notifyObservers();
    }

    /**
     * Modifie le score maximal.
     * @param maxScore le nouveau score maximal.
     */
    public void setMaxScore(int maxScore){
	this.maxScore = maxScore;
	setChanged();
	notifyObservers();
    }

    /**
     * Spcifie si la reponse est juste ou non.
     * @param b le boolean indiquant si la reponse est juste ou non.
     */
    public void setEval(boolean b){
	this.eval = b;
    }

    /**
     * Spcifie les leons.
     * @param lessons les leons  spcifier.
     */
    public void setLessons(ArrayList<Lesson> lessons){
	this.lessons = lessons;
	setChanged();
	notifyObservers();
    }

    /**
     * Spcifie les leons coches et le nombre de mot de ces leons.
     */
    public void setChecked(){
	checkedWordsCount = 0; 
	getLessonsChecked().clear();
	for (Lesson l : getLessons()){
	    if (l.isChecked()){
		addLessonChecked(new Lesson(l));
		checkedWordsCount += l.getLength();
	    }
	}
	setChanged();
	notifyObservers();
    }

    /**
     * Coche ou non toutes les leons.
     * @param b indique si on coche toutes les leons ou non.
     */
    public void checkAll(boolean b){
	getLessonsChecked().clear();
	lessonsCheckedCopy.clear();
	for (Lesson l : getLessons())
	    l.check(b);
	setChecked();
	setMaxScore();
    }

    /**
     * Ajoute une leon.
     * @param l la leon  ajouter.
     */
    public void addLesson(Lesson l){
	lessons.add(l);
	setChanged();
	notifyObservers();
    }

    /**
     * Ajoute une leon coche.
     * @param l la leon  ajouter.
     */
    public void addLessonChecked(Lesson l){
	lessonsChecked.add(l);
	setChanged();
	notifyObservers();
    }

    /**
     * Remplace une leon.
     * @param index l'indice de la leon  remplacer.
     * @param l la leon  ajouter.
     */
    public void setLesson(int index, Lesson l){
	lessons.add(index, l);
	setChanged();
	notifyObservers();
    }

    /**
     * Retire une leon.
     * @param index l'indice de la leon  retirer.
     */
    public void removeLesson(int index){
	lessons.remove(index);
	setChanged();
	notifyObservers();
    }

    /**
     * Active le reasking ou non.
     * @param b pour activer reasking ou non.
     */
    public boolean setReasking(boolean b){
	return reasking = b;
    }

   /**
     * Spcifie le nombre de reponses donnes.
     * @param n le nombre de rponses donnes. 
     */
    public void setAsked(int n){
	asked = n;
    }

   /**
     * Spcifie le nombre d'erreur.
     * @param n le nombre d'erreurs. 
     */
    public void setErrors(int n){
	errors = n;
    }

    /**
     * Retire une leon coche.
     * @param index l'indice de la leon  retirer.
     */
    public void removeLessonChecked(int index){
	lessonsChecked.remove(index);
	setChanged();
	notifyObservers();
    }

    /**
     * Retire une leon coche.
     * @param index l'indice de la leon  retirer.
     */
    public void copy(){
	lessonsCheckedCopy = new ArrayList<Lesson>();
	for (Lesson l : getLessonsChecked()){
	    Lesson k = new Lesson(new String(l.getName()));
	    Iterator eng = l.getEngWords().iterator();
	    Iterator fr = l.getFrWords().iterator();
	    while (eng.hasNext() && fr.hasNext())	
		k.addWord(new String((String)eng.next()), new String((String)fr.next()));
	    lessonsCheckedCopy.add(k);
	}    
    }

    /**
     * Spcifie la langue d'interrogation du test.
     * @param i la langue (ENG ou FR).
     */
    public void setLang(int i){
	lang = i;
    }

    /**
     * Spcifie la chane de caractres saisie par l'utilisateur.
     * @param s la chane.
     */
    public void setInput(String s){
	input = s;
    }

  /**
     * Spcifie la chane de caractres  afficher en sortie.
     * @param s la chane.
     */
    public void setOutput(String s){
	output = s;
    }

    /**
     * Termine ou non le test.
     * @param b le boolean indiquant si on termine le test ou non.
     */
    public void setFinished(boolean b){
       finished = b;
       setChanged();
       notifyObservers();      
    }

    /**
     * Specifie la leon.
     * @param l la leon
     */
    public void setLesson(String l){
	lesson = l;
    }

    /**
     * Charge leons contenus dans un fichier texte.
     * 
     * Le fichier texte doit respecter le format suivant:
     * Le nom de la leon est sur une ligne, puis sur les lignes suivantes
     * figurent les mots de vocabulaires sous la forme anglais=francais.
     * Une ligne vide spare les leons.
     *
     * Exemple :
     *
     * NomLesson1
     * motAnglais=motFrancais
     * motAnglais2=motFrancais2
     * 
     * NomLesson2 
     * motAnglais=motFrancais
     *
     * NomLesson3
     * motAnglais=motFrancais
     * motAnglais3=motFrancais3
     * motAnglais3=motFrancais3
     *
     * @param fileName le nom du fichier (qui peut tre un chemin) contenant les leons.
     */
    public void load(String fileName) throws LoadException{
	getLessons().clear(); 
	getLessonsChecked().clear();
	getLessonsCheckedCopy().clear();
	wordsCount = 0;
	checkedWordsCount = 0;
	BufferedReader in ;
	int i;
	Lesson l = null;
	try { 
	    in = new BufferedReader(new FileReader(fileName)) ;
	    String line = in.readLine() ;
	    while (line != null) {
		while (! line.equals(lessonSeparator)){
		    i = line.lastIndexOf(wordSeparator);
		    if (i == -1)
			l = new Lesson(line);
		    else
			l.addWord(line.substring(0,i),line.substring(i + 1, line.length()));		    
		    line = in.readLine();
		    if (line == null)
			break;
		}
		addLesson(l);
		wordsCount += l.getLength();
		if (l.isChecked()){
		    addLessonChecked(l);
		    checkedWordsCount += l.getLength();
		}	    
		setMaxScore(getMaxScore() + l.getLength());
		line = in.readLine() ;
	    }//while
	in.close();
	}	    
	catch (IOException e) {
	    e.getMessage();
	}//try - catch
	if (getWordsCount() == 0)
	    throw new LoadException();
	setChanged();
	notifyObservers();
    }

    /**
     * Sauvegarde les leons dans un fichier texte.
     * @param fileName le nom du fichier(qui peut tre un chemin) pour la sauvegarde.
     */    
    public void save(String fileName){
	BufferedWriter out;
	int i = 0;
	try { 
	    out = new BufferedWriter(new FileWriter(fileName));
	    for (Lesson l: getLessons()){
		out.write(l.getName());
		out.newLine();
		i = 0;
		Iterator eng = l.getEngWords().iterator();
		Iterator fr = l.getFrWords().iterator();
		while (eng.hasNext() && fr.hasNext()){
		    out.write(eng.next()+ "=" + fr.next());
		    out.newLine();
		}
		out.newLine();
	    }
	out.close();
	}	    
	catch (IOException e) {
	    e.getMessage();
	}	    
    }
	    
    /**
     * Lance le test. Inutilise - ne sert qu'en mode "terminal".
     */
    public void run(){
	String input="";
 	BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
	do {
	    try{
		System.out.println("Chose the mode:\nenglish->french: f\nfrench->english : e");

		input = in.readLine();
	
	    }
	    catch(IOException e){
		e.getMessage();
	    }
	}
	while(!input.equals("f") && !input.equals("e") );
	int lang = input.equals("e")?ENG:FR;
	boolean english = (lang == 0);
	String word; 
	boolean eval = false;
	int randomLesson;
	while (! isFinished()){
	    randomLesson = (int)(Math.random() * getCheckedLength());
	    Lesson l = getLessonChecked(randomLesson);
	    int randomWord = l.getRandomIndex();
	    try{
		word = english?l.getWord(FR,randomWord):l.getWord(ENG,randomWord);
		String question = english?"The english for \"" + word +  "\" is :":"\""+word+"\"" + " means :";		
		System.out.println(l.getName() + "\n" + question);
		eval = l.compare(lang, randomWord, in.readLine());
	    }
	    catch(IOException e){
		e.getMessage();
	    }
	    if (eval){
		setScore(getScore() +  1);
		l.removeWord(randomWord);
	    }
	    else {
		System.out.println("---WRONG : the correct answer is " + l.getWord(lang,randomWord)+"\n");
		if ( ! getReasking())
		    l.removeWord(randomWord);
	    }
	    if(l.isEmpty())
		removeLessonChecked(randomLesson);
	    System.out.println();
	}
	System.out.println("Your score : " + getScore() + "/" + getMaxScore());	
    }

    /**
     * Soummet une question  l'utilisateur.
     * Une leon est choisie alatoirement parmi les leons coches, puis un mot est choisi
     * alatoirement dans cette leon.
     */
    public void question(){ 
	if (! getLessonsCheckedCopy().isEmpty() && ! isFinished()){
	    boolean english = (getLang() == 0);
	    String word; 
	    randomLessonIndex = (int)(Math.random() * lessonsCheckedCopy.size());
	    randomLesson = lessonsCheckedCopy.get(randomLessonIndex);
	    randomWord = randomLesson.getRandomIndex();
	
	    word = english?randomLesson.getWord(FR,randomWord):randomLesson.getWord(ENG,randomWord);
	    String question = english?"The english for \"" + word +  "\" is :":"\""+word+"\"" + " means :";
	    setLesson(randomLesson.getName() + "\n");
	    setOutput(question);
	    }
    }

    /**
     * Analyse la rponse de l'utilisateur.
     */		
    public void answer(){
	if (! getLessonsCheckedCopy().isEmpty() && !isFinished()){
	    boolean eval = randomLesson.compare(lang, randomWord, getInput());
	    setEval(eval);
	    setMaxScore(getMaxScore()+1);
	    if (eval){
		setScore(getScore()+ 1);
		setOutput("\n");
		randomLesson.removeWord(randomWord);
		setAsked(getAsked()+1);
	    }
	    else{
		setOutput(randomLesson.getWord(lang,randomWord));
		setErrors(getErrors()+1);
		if ( ! getReasking()){
		    setAsked(getAsked()+1);
		    randomLesson.removeWord(randomWord);
		}
	    }
	    
	    if(randomLesson.isEmpty())
		lessonsCheckedCopy.remove(randomLessonIndex);
	    if(getLessonsCheckedCopy().isEmpty())
		end();
	}
	else
	    end();		    
    }

    /**
     * Termine le test et affiche le score.
     */	
    public void end(){
	setOutput("\n\nYour score is : " + getScore() + "/" + getMaxScore()+"\n\n\n");
	setFinished(true);
	setScore(0);
	setMaxScore(0);
	setAsked(0);
	setErrors(0);
	//setEval(false);
    }
}//class Test
