package mtv.gui;

import static mtv.core.Lesson.LANG_COUNT;
import mtv.core.Test;
import mtv.core.Lesson;
import java.util.ArrayList;
import javax.swing.table.AbstractTableModel;

/**
 * Classe reprsentant le modle de la jtable reprsentant les leons.
 *
 * @author Christopher Henard
 * @version 27/10/2008
 */

public class LessonsTableModel extends AbstractTableModel{

    //CHAMPS------------------------------------------------------
    private Test test; //modle
    private String[] columnNames = {"Lesson","Selection"};



    //CONSTRUCTEUR-----------------------------------------------
    /**
     * Construit le modle de jtable  partir du test.
     * @param t le modele (test).
     */
    public LessonsTableModel(Test t){
	super();
	test = t;
    
    }



    //METHODES D'OBSERVATION ET DE TRANSFORMATION------------------
    /**
     * Retourne le nombre de colonnes.
     * @return le nombre de colonnes que contiendra la jtable constuite sur ce modle.
     */
    public int getColumnCount() {
        return columnNames.length;
    }

    /**
     * Retourne le nombre de lignes.
     * @return le nombre de lignes que contiendra la jtable constuite sur ce modle.
     */
    public int getRowCount() {
        return test.getLength();
    }

    /**
     * Retourne le titre de la colonne numro col.
     * @return le titre de la colonne numro col.
     */
    public String getColumnName(int col) {
        return columnNames[col];
    }

    /**
     * Retourne l'objet ligne row et colonne col.
     * @return l'objet ligne row et colonne col.
     */
    public Object getValueAt(int row, int col) {
	if (col == 0)
	    return test.getLesson(row).getName();
	else
	    return new Boolean(test.getLesson(row).isChecked());
    }

    /*
     * JTable utilise cette mthode pour dterminer le default renderer
     * de chaque cellule. Sans cette mthode on aurait pas les checkbox dans la
     * deuxieme colonne.
     */
    public Class getColumnClass(int c) {
        return getValueAt(0, c).getClass();
    }

    /*
     * Retourne true ou false selon que la cellule (row,col) est ditable ou non.
     * @return un boolean indiquant si la cellule (row,col) est ditable ou non.
     */
    public boolean isCellEditable(int row, int col) {
	return true;
    }

    /*
     * Spcifie la valeur de la cellule(row,col) par l'objet value.
     * @param value l'objet, la valeur de la cellule.
     * @param row la ligne de la cellule.
     * @param col la colonne de la cellule.
     */
    public void setValueAt(Object value, int row, int col) {
	if (col==1){
	    test.getLesson(row).check(!test.getLesson(row).isChecked());
	}
	else{
	    test.getLesson(row).setName((String)value);
	}
	test.setChecked();	    
    }
}//class LessonsTableModel
