package mtv.gui;

import mtv.core.Test;
import java.util.Observer;
import java.util.Observable;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JCheckBox;
import javax.swing.table.AbstractTableModel;
import javax.swing.border.TitledBorder;
import static javax.swing.SwingUtilities.*;
import static javax.swing.SwingUtilities.RIGHT;
import static mtv.core.Lesson.ENG; 
import static mtv.core.Lesson.FR; 
import static java.awt.Color.red;

/**
 * Classe reprsentant la vue des leons.
 *
 * @author Christopher Henard
 * @version 27/10/2008
 */

public class LessonsView extends JPanel implements Observer {

    //CHAMPS------------------------------------------------------
    private Test test; //modle
    private JTable lessonsTable;
    private JScrollPane scrollPane;
    private JCheckBox selectAll;
    private JCheckBox removeAll;
    private JCheckBox frenchToEnglish;
    private JCheckBox englishToFrench;
    private JCheckBox reasking;
    private JLabel wordsCount;
    private JLabel lessonsCount;
    private JPanel bottom;



    //CONSTRUCTEUR-----------------------------------------------
    /**
     * Construit la vue des leons  partir du test.
     * @param t le modele (test).
     */
    public LessonsView(Test t){
	super(new BorderLayout());
	setBorder(new TitledBorder("Lessons"));  // bordure de panneau
	test = t;
	lessonsTable = new JTable(new LessonsTableModel(test));
	lessonsTable.getColumnModel().getColumn(0).setPreferredWidth(150);;
	lessonsTable.getColumnModel().getColumn(1).setPreferredWidth(50);
        lessonsTable.setPreferredScrollableViewportSize(new Dimension(200, 200));
        lessonsTable.setFillsViewportHeight(true);
	lessonsTable.setSelectionForeground(red);
	scrollPane = new JScrollPane(lessonsTable);
	selectAll = new JCheckBox("Select all", true);
	selectAll.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    if (selectAll.isSelected() && !test.allChecked())
			test.checkAll(true);	 
		}
	    });
	removeAll = new JCheckBox("Remove all", false);
	removeAll.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    if (removeAll.isSelected() && test.getCheckedLength() != 0)
			test.checkAll(false);	 
		}
	    });
	frenchToEnglish = new JCheckBox("French to English", true);
	frenchToEnglish.addItemListener(new ItemListener(){
		public void itemStateChanged(ItemEvent e) {
		    if (e.getStateChange() == ItemEvent.SELECTED){
			test.setLang(ENG); 
			englishToFrench.setSelected(false);
			
		    }
		    else
			englishToFrench.setSelected(true);
		}
	    });
	englishToFrench = new JCheckBox("English to French",false);
	englishToFrench.addItemListener(new ItemListener(){
		public void itemStateChanged(ItemEvent e) {
		    if (e.getStateChange() == ItemEvent.SELECTED){
			test.setLang(FR);
			frenchToEnglish.setSelected(false);
		    }
		    else
			frenchToEnglish.setSelected(true);	
		}
	    });
	reasking = new JCheckBox("Reask wrong words", true);
	reasking.addItemListener(new ItemListener(){
		public void itemStateChanged(ItemEvent e) {
		    test.setReasking(e.getStateChange() == ItemEvent.SELECTED);		    
		}
	    });
	lessonsCount = new JLabel("  Lessons count : "+test.getCheckedLength());
	wordsCount = new JLabel("  Words count : "+test.getCheckedWordsCount());
	add(scrollPane,BorderLayout.CENTER);
	bottom = new JPanel(new GridLayout(5,2,5,0));
	bottom.add(selectAll);
	bottom.add(frenchToEnglish);
	bottom.add(removeAll);
	bottom.add(englishToFrench);
	bottom.add(reasking);
	bottom.add(new JLabel(""));
	bottom.add(new JLabel(""));
	bottom.add(new JLabel(""));
	bottom.add(lessonsCount);
	bottom.add(wordsCount);
	add(bottom,BorderLayout.SOUTH);
	test.addObserver(this); //se signale comme vue au modle.
    }

    /**
     * Mthode de l'interface Observer, appele pour mettre  jour la vue.
     * @param o le modle (objet observ)
     * @param arg l'argument transmis par le modele (quand ses donnees changent) 
     * en argument de notifyObservers() (non utilis ici).
     */
    public void update(Observable o, Object arg){
	update();
    }
    
    /**
     * Met  jour la vue.
     */
    public void update(){
	Runnable code = new Runnable() {
		public void run(){
		    updateUI();
		    if (!test.allChecked())
			selectAll.setSelected(false);
		    if (test.getCheckedLength() != 0)
			removeAll.setSelected(false);
		    
		    wordsCount.setText("Words count : " + test.getCheckedWordsCount());
		    lessonsCount.setText("Lessons count : "+ test.getCheckedLength());

		}
	    };
	if (isEventDispatchThread())
	    code.run();
	else
	    invokeLater(code);    
    }
}//class LessonView
    
    
    
