package mtv.gui;

import mtv.core.Test;
import mtv.core.LoadException;
import java.io.File;
import java.util.Observer;
import java.util.Observable;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import static javax.swing.SwingUtilities.*;

/**
 * Classe crant le menu de notre GUI.
 *
 * @author Christopher Henard
 * @version 27/10/2008
 */

public class Menu extends JMenuBar implements Observer{

    //CHAMPS ET CONSTANTES---------------------------------------
    private Test test;
    private static JMenu file, tesT, help;
    private static JMenuItem load, save, exit, takeTest, stopTest, clearOutput, howToUse, about;



    //CONSTRUCTEUR-----------------------------------------------
    /**
     * Construit le menu  partir du modle test.
     * @param t le modele (test).
     */
    public Menu(Test t){
	test = t;
	file = new JMenu("File");
	load = new JMenuItem("Open File...");
	save = new JMenuItem("Save As...");
	exit = new JMenuItem("Exit MTV");
	tesT = new JMenu("Test");
	takeTest = new JMenuItem("Take the Test");
	stopTest = new JMenuItem("Stop the Test");
	clearOutput = new JMenuItem("Clear Output");
	help = new JMenu("Help");
	howToUse = new JMenuItem("How to Use");
	about = new JMenuItem("About MTV...");
	stopTest.setEnabled(false);
	load.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    JFileChooser jfc = new JFileChooser();
		    if (jfc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
			File f = jfc.getSelectedFile();
			try {
			    test.load(f.getPath());
			}
			catch(LoadException e){
			    JOptionPane.showMessageDialog(Menu.this.getParent(),"Please read \"How to Use\" in Help menu.", 
			    "Incorrect File Format",JOptionPane.ERROR_MESSAGE);
			}
		    }
		}
	    });
	save.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    JFileChooser jfc = new JFileChooser();
		    if (jfc.showSaveDialog(Menu.this.getParent()) == JFileChooser.APPROVE_OPTION) {
			File f = jfc.getSelectedFile();
			test.save(f.getPath());
		    }
		}
	    });
	takeTest.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    ToolBar.testRunning();
		    testRunning();
		}
	    });
	stopTest.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    ToolBar.testFinished();
		}
	    });
	clearOutput.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    TestView.output.setText("");
		    TestView.pos = 0;
		}
	    });
	exit.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    System.exit(0);
		}
	    });
	howToUse.addActionListener(new ActionListener(){
	    public void actionPerformed(ActionEvent e){
		JOptionPane.showMessageDialog(Menu.this.getParent(),
		    "First, use default vocabulary list mtv.txt (which is loaded automatically if"+
                    " the jar and the mtv.txt files\nare situated in the same directory, otherwise load " +
                    "vocabulary list with the load file button).\n"+
		    "Then, select your preferred settings on the settings panel and take the test "+
		    "by pressing the play button.\n"+
		    "Enter your answers in the input area and press the enter key to submit.\n"+
		    "You can stop the test at any time by pressing the stop button.\n\n"+			      
		    "From the settings panel you can modify words or lessons names from the tables and then save your\n"+
		    "modified vocabulary list to a file.\n\n"+
		    "In order to load vocabulary list from .txt files, remember that files must have that format:\n\n"+
		    "Lesson1Name\nenglishWord1=frenchWord1\nenglishWord2=frenchWord2\n\nLesson2Name"+
		    "\nenglishWord1=frenchWord1\nenglishWord2=frenchWord2\nenglishWord3=frenchWord3"+
		    "\n\n and so on. Exactly like that, no spaces between words and equal sign."+
		    "\n\n\n Have fun.","How To Use",JOptionPane.INFORMATION_MESSAGE);
		}
	    });
	about.addActionListener(new ActionListener(){
	    public void actionPerformed(ActionEvent e){
		JOptionPane.showMessageDialog(Menu.this.getParent(),
		    "\n MTV - Minimum Toeic Vocabulary \n version 0.3 (11/2008)"
		    + "\n Christopher Henard", "about MTV...",JOptionPane.INFORMATION_MESSAGE);
		}
	    });
	exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.ALT_MASK));
	takeTest.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.ALT_MASK));
	stopTest.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.ALT_MASK));
	clearOutput.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.ALT_MASK));
	file.add(load);
	file.add(save);
	file.addSeparator();
	file.add(exit);
	file.setMnemonic(KeyEvent.VK_F);
	tesT.add(takeTest);
	tesT.add(stopTest);
	tesT.addSeparator();
	tesT.add(clearOutput);
	tesT.setMnemonic(KeyEvent.VK_T);
	help.add(howToUse);
	help.addSeparator();
	help.add(about);
	help.setMnemonic(KeyEvent.VK_H);
	add(file);
	add(tesT);
	add(help);
	test.addObserver(this); //se signale comme vue au modle.
    }	

    /**
     * Dsactive ou active les lments du menu qui doivent tre accessibles
     * (ou non) lors du test.
     */
    public static void testRunning(){
	save.setEnabled(false);
	load.setEnabled(false);
	help.setEnabled(false);
	takeTest.setEnabled(false);
	clearOutput.setEnabled(false);
	stopTest.setEnabled(true);
    }

    /**
     * Mthode de l'interface Observer, appele pour mettre  jour la vue.
     * @param o le modle (objet observ)
     * @param arg l'argument transmis par le modele (quand ses donnees changent) 
     * en argument de notifyObservers() (non utilis ici).
     */
    public void update(Observable o, Object arg){
	Runnable code = new Runnable() {
		public void run(){
		    if (test.isFinished()){
			load.setEnabled(true);
			save.setEnabled(true);
			takeTest.setEnabled(true);
			help.setEnabled(true);
			clearOutput.setEnabled(true);
			stopTest.setEnabled(false);
			Interface.tabbedPane.setEnabledAt(0,true);
		    }
		}
	    };
	if (isEventDispatchThread())
	    code.run();
	else
	    invokeLater(code);    
    }
}//class Menu
