package mtv.gui;

import mtv.core.Test;
import mtv.core.LoadException;
import java.io.File;
import java.util.Observer;
import java.util.Observable;
import java.net.URL;
import javax.swing.JOptionPane;
import javax.swing.JToolBar;
import javax.swing.JButton;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import static javax.swing.SwingUtilities.*;

/**
 * Classe crant la barre d'outils de l'interface homme machine.
 *
 * @author Christopher Henard
 * @version 27/10/2008
 */
public class ToolBar extends JToolBar implements Observer{

    //CHAMPS ET CONSTANTES---------------------------------------
    private static Test test;
    private static JButton bload, bsave, brun, bstop, brefresh, bquit;
    private ImageIcon load, save, run, stop, refresh, quit;
    private final URL urlload = getClass().getResource("icons/load.png");
    private final URL urlsave = getClass().getResource("icons/save.png");
    private final URL urlrun = getClass().getResource("icons/run.png");
    private final URL urlstop = getClass().getResource("icons/stop.png");
    private final URL urlrefresh = getClass().getResource("icons/refresh.png");
    private final URL urlquit = getClass().getResource("icons/quit.png");


    //CONSTRUCTEUR-----------------------------------------------
    /**
     * Construit la toolbar  partir du modle test.
     * @param t le modele (test).
     */
    public ToolBar(Test t){
	super();
	test = t;
	setFloatable(false);
	if (urlload != null) load = new ImageIcon(urlload);
	if (urlsave != null) save = new ImageIcon(urlsave);
	if (urlrun != null) run = new ImageIcon(urlrun);
	if (urlstop != null) stop = new ImageIcon(urlstop);
	if (urlrefresh != null) refresh = new ImageIcon(urlrefresh);
	if (urlquit != null) quit = new ImageIcon(urlquit);
	bload = new JButton(load);
	bload.setToolTipText("Load file containing vocabulary");
	bsave = new JButton(save);
	bsave.setToolTipText("Save current vocabulary to .txt file");
	brun = new JButton(run);
	brun.setToolTipText("Take the test");
	bstop = new JButton(stop);
	bstop.setToolTipText("Stop the test");
	bstop.setEnabled(false);
	brefresh = new JButton(refresh);
	brefresh.setToolTipText("Clear the output field");
	bquit = new JButton(quit);
	bquit.setToolTipText("Exit");
	bload.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    JFileChooser jfc = new JFileChooser();
		    if (jfc.showOpenDialog(ToolBar.this.getParent()) == JFileChooser.APPROVE_OPTION) {
			File f = jfc.getSelectedFile();
			try {
			    test.load(f.getPath());
			}
			catch(LoadException e){
			    JOptionPane.showMessageDialog(ToolBar.this.getParent(),"Incorrect file format. "+
			    "Please read \"How to Use\" in Help menu.", 
			    "Incorrect File Format",JOptionPane.ERROR_MESSAGE);
			}
		    }
		}
	    });
	bsave.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    JFileChooser jfc = new JFileChooser();
		    if (jfc.showSaveDialog(ToolBar.this.getParent()) == JFileChooser.APPROVE_OPTION) {
			File f = jfc.getSelectedFile();
			test.save(f.getPath());
		    }
		}
	    });
	brun.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    if (test.getCheckedWordsCount() > 0 ){
			Menu.testRunning();
			testRunning();
		    }
		    else 
			JOptionPane.showMessageDialog(ToolBar.this.getParent(), "No lessons selected.", "Error", JOptionPane.ERROR_MESSAGE); 		    
		}
	    });
	bstop.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    testFinished();
		}
	    });
	brefresh.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    TestView.output.setText("");
		}
	    });
	bquit.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    System.exit(0);
		}
	    });	
	add(bload);
	add(bsave);
	add(brun);
	add(bstop);
	add(brefresh);
	add(bquit);
	test.addObserver(this);
	
    }

    /**
     * Dsactive ou active les lments de la toolbar qui doivent tre accessibles
     * (ou non) lors du test.
     */
    public static void testRunning(){
	Interface.tabbedPane.setEnabledAt(0,false);
	Interface.tabbedPane.setSelectedIndex(1);
	test.setFinished(false);
	test.copy();
	test.question();
	test.setMaxScore(0);
	bload.setEnabled(false);
	bsave.setEnabled(false);
	brun.setEnabled(false);
	brefresh.setEnabled(false);
	bstop.setEnabled(true);
    }

    /**
     * Active le panel settings et met fin au test.
     */
    public static void testFinished(){
	test.setFinished(true);
	Interface.tabbedPane.setEnabledAt(0,true);
	test.end();
    }
	
    /**
     * Mthode de l'interface Observer, appele pour mettre  jour la vue.
     * @param o le modle (objet observ)
     * @param arg l'argument transmis par le modele (quand ses donnees changent) 
     * en argument de notifyObservers() (non utilis ici).
     */	
    public void update(Observable o, Object arg){
	Runnable code = new Runnable() {
		public void run(){
		    if (test.isFinished()){
			bload.setEnabled(true);
			bsave.setEnabled(true);
			brun.setEnabled(true);
			brefresh.setEnabled(true);
			bstop.setEnabled(false);
			Interface.tabbedPane.setEnabledAt(0,true);
		    }
		}
	    };
	if (isEventDispatchThread())
	    code.run();
	else
	    invokeLater(code);	    
    }
}//class ToolBar
 
    
    


