package mtv.gui;

import static mtv.core.Lesson.LANG_COUNT;
import mtv.core.Test;
import mtv.core.Lesson;
import java.util.ArrayList;
import javax.swing.table.AbstractTableModel;

/**
 * Classe reprsentant le modle de la jtable reprsentant les mots des leons coches.
 *
 * @author Christopher Henard
 * @version 27/10/2008
 */

public class WordsTableModel extends AbstractTableModel{
    
    //CHAMPS------------------------------------------------------
    private Test test;
    private String[] columnNames = {"English","French"};



    //CONSTRUCTEUR-----------------------------------------------
    /**
     * Construit le modle de jtable  partir du test.
     * @param t le modele (test).
     */
    public WordsTableModel(Test t){
	test = t;
    }



    //METHODES D'OBSERVATION ET DE TRANSFORMATION------------------
    /**
     * Retourne le nombre de colonnes.
     * @return le nombre de colonnes que contiendra la jtable constuite sur ce modle.
     */
    public int getColumnCount() {
        return columnNames.length;
    }

    /**
     * Retourne le nombre de lignes.
     * @return le nombre de lignes que contiendra la jtable constuite sur ce modle.
     */
    public int getRowCount() {
        return test.getCheckedWordsCount();
    }

    /**
     * Retourne le titre de la colonne numro col.
     * @return le titre de la colonne numro col.
     */
    public String getColumnName(int col) {
        return columnNames[col];
    }

    /**
     * Retourne l'objet ligne row et colonne col.
     * @return l'objet ligne row et colonne col.
     */
    public Object getValueAt(int row, int col) {
	int i = 0;
	int currentCount = 0;

	while (row >= currentCount+test.getLessonChecked(i).getLength()){
	
	    currentCount += test.getLessonChecked(i).getLength();
	    i++;
	}

	return test.getLessonChecked(i).getWord(col, row - currentCount);	    
    }

    /*
     * Retourne true ou false selon que la cellule (row,col) est ditable ou non.
     * @return un boolean indiquant si la cellule (row,col) est ditable ou non.
     */
    public boolean isCellEditable(int row, int col) {
	return true;
    }

    /*
     * Spcifie la valeur de la cellule(row,col) par l'objet value.
     * @param value l'objet, la valeur de la cellule.
     * @param row la ligne de la cellule.
     * @param col la colonne de la cellule.
     */
    public void setValueAt(Object value, int row, int col) {

	String word = (String)value;

	int i = 0;
	int currentCount = 0;
	while (row >= currentCount+test.getLessonChecked(i).getLength()){
	
	    currentCount += test.getLessonChecked(i).getLength();
	    i++;
	}
	test.getLessonChecked(i).setWord(col, row - currentCount,word);
    }
}//class WordsTableModel
