package mtv.gui;

import mtv.core.Test;
import java.util.Observer;
import java.util.Observable;
import java.awt.Dimension;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.table.AbstractTableModel;
import javax.swing.border.TitledBorder;
import static javax.swing.SwingUtilities.*;
import static java.awt.Color.red;

/**
 * Classe reprsentant la vue des mots.
 *
 * @author Christopher Henard
 * @version 27/10/2008
 */
public class WordsView extends JPanel implements Observer {

    //CHAMPS------------------------------------------------------
    private Test test; //modele
    private JTable wordsTable;
    private JScrollPane scrollPane;



    //CONSTRUCTEUR-----------------------------------------------
    /**
     * Construit la vue des mots  partir du test.
     * @param t le modele (test).
     */
    public WordsView(Test t){
	super();
	setBorder(new TitledBorder("Words"));  // bordure de panneau
	test = t;
	wordsTable = new JTable(new WordsTableModel(test));
	wordsTable.getColumnModel().getColumn(0).setPreferredWidth(150);;
	wordsTable.getColumnModel().getColumn(1).setPreferredWidth(150);
        wordsTable.setPreferredScrollableViewportSize(new Dimension(300, 150));
        wordsTable.setFillsViewportHeight(true);
	wordsTable.setSelectionForeground(red);
	scrollPane = new JScrollPane(wordsTable);
	add(scrollPane);
	test.addObserver(this);
    }

    /**
     * Mthode de l'interface Observer, appele pour mettre  jour la vue.
     * @param o le modle (objet observ)
     * @param arg l'argument transmis par le modele (quand ses donnees changent) 
     * en argument de notifyObservers() (non utilis ici).
     */
    public void update(Observable o, Object arg){
	update();
    }

    /**
     * Met  jour la vue.
     */
    public void update(){
	Runnable code = new Runnable() {
		public void run(){
		    wordsTable.updateUI();
		}
	    };
	if (isEventDispatchThread())
	    code.run();
	else
	    invokeLater(code);    
    }
}//class WordsView
  
